package cn.com.infosec.crypto.engines;

import cn.com.infosec.crypto.BlockCipher;
import cn.com.infosec.crypto.CipherParameters;
import cn.com.infosec.crypto.params.RC5Parameters;

public class RC564Engine implements BlockCipher {
   private static final int wordSize = 64;
   private static final int bytesPerWord = 8;
   private static final long P64 = -5196783011329398165L;
   private static final long Q64 = -7046029254386353131L;
   private int _noRounds = 12;
   private long[] _S = null;
   private boolean forEncryption;

   private void setKey(byte[] key) {
      int var2;
      long[] var3 = new long[var2 = (key.length + 7) / 8];

      int var4;
      int var5;
      long var6;
      for(var4 = 0; var4 != key.length; ++var4) {
         var6 = var3[var5 = var4 / 8] + ((long)(key[var4] & 255) << var4 % 8 * 8);
         var3[var5] = var6;
      }

      (this._S = new long[(this._noRounds + 1) * 2])[0] = -5196783011329398165L;

      int key1;
      long[] var14;
      for(key1 = 1; key1 < (var14 = this._S).length; ++key1) {
         var14[key1] = var14[key1 - 1] + -7046029254386353131L;
      }

      if (var2 > var14.length) {
         key1 = var2 * 3;
         var5 = key1;
      } else {
         key1 = var14.length * 3;
         var5 = key1;
      }

      var6 = 0L;
      long var8 = 0L;
      key1 = 0;
      var4 = 0;

      for(int var10 = 0; var10 < var5; ++var10) {
         long[] var11;
         long var10005 = var6 = this.rotateLeft((var11 = this._S)[key1] + var6 + var8, 3L);
         long var10006 = var8;
         var11[key1] = var6;
         var8 += var3[var4] + var6;
         long var15 = var10005 + var10006;
         var3[var4] = var8 = this.rotateLeft(var8, var15);
         key1 = (key1 + 1) % this._S.length;
         var4 = (var4 + 1) % var2;
      }

   }

   private int encryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      long in1 = this.bytesToWord(in, inOff) + this._S[0];
      long var5 = this.bytesToWord(in, inOff + 8) + this._S[1];

      long var10000;
      for(int var7 = 1; var7 <= this._noRounds; in1 = var10000) {
         int in2;
         in1 = this.rotateLeft(var5 ^ (var5 = this.rotateLeft(in1 ^ var5, var5) + this._S[in2 = var7 * 2]), var5) + this._S[in2 + 1];
         ++var7;
         var10000 = var5;
         var5 = in1;
      }

      this.wordToBytes(in1, out, outOff);
      int var8 = outOff + 8;
      this.wordToBytes(var5, out, var8);
      return 16;
   }

   private int decryptBlock(byte[] in, int inOff, byte[] out, int outOff) {
      long var5 = this.bytesToWord(in, inOff);
      long in1 = this.bytesToWord(in, inOff + 8);

      long var10000;
      for(int var7 = this._noRounds; var7 >= 1; in1 = var10000) {
         long var10001 = var5;
         int in2;
         var5 ^= this.rotateRight(in1 - this._S[(in2 = var7 * 2) + 1], var5);
         in1 = this.rotateRight(var10001 - this._S[in2], var5) ^ var5;
         --var7;
         var10000 = var5;
         var5 = in1;
      }

      this.wordToBytes(var5 - this._S[0], out, outOff);
      this.wordToBytes(in1 - this._S[1], out, outOff + 8);
      return 16;
   }

   private long rotateLeft(long x, long y) {
      return x << (int)(y &= 63L) | x >>> (int)(64L - y);
   }

   private long rotateRight(long x, long y) {
      return x >>> (int)(y &= 63L) | x << (int)(64L - y);
   }

   private long bytesToWord(byte[] src, int srcOff) {
      long var3 = 0L;

      for(int var5 = 7; var5 >= 0; --var5) {
         var3 = (var3 << 8) + (long)(src[var5 + srcOff] & 255);
      }

      return var3;
   }

   private void wordToBytes(long word, byte[] dst, int dstOff) {
      for(int var5 = 0; var5 < 8; ++var5) {
         dst[var5 + dstOff] = (byte)((int)word);
         word >>>= 8;
      }

   }

   public String getAlgorithmName() {
      return "RC5-64";
   }

   public int getBlockSize() {
      return 16;
   }

   public void init(boolean forEncryption, CipherParameters params) {
      if (params instanceof RC5Parameters) {
         RC564Engine var10000 = this;
         RC564Engine var10001 = this;
         RC564Engine var10002 = this;
         RC5Parameters var3 = (RC5Parameters)params;
         var10002.forEncryption = forEncryption;
         var10001._noRounds = var3.getRounds();
         var10000.setKey(var3.getKey());
      } else {
         throw new IllegalArgumentException("invalid parameter passed to RC564 init - " + params.getClass().getName());
      }
   }

   public int processBlock(byte[] in, int inOff, byte[] out, int outOff) {
      return this.forEncryption ? this.encryptBlock(in, inOff, out, outOff) : this.decryptBlock(in, inOff, out, outOff);
   }

   public void reset() {
   }
}
